/*
	Olimexino-328 + MOD-RFID1356-BOX example.

	Your Olimexino328 board shoud be connected to a power source
	different from USB because, if present, the USB connection
	takes up the D0 and D1 pins needed for the serial communication
	via UEXT. (You could connect it, but there will probably be a 
	lot of noise and the RFID tag numbers won't be accurate)
	
	This example program blinks the LEDs twice if it detects
	an RFID tag.

	If you have any questions, email
	support@olimex.com

	OLIMEX 2013
	http://www.olimex.com/dev

*/

#include <string.h>

//The MOD-RFID1356-BOX Serial Communication baud rate
#define rf_baud 57600

//Defs specific to the Olimexino-328 board
#define BBIT (PIND & B00000100)!=0			// Used to check if button has been pressed
#define BUTTONINPUT DDRD &= B11111011		// Init the Button
#define YLED 9 
#define GLED 13

#define YELLOW 0
#define GREEN  1
#define BOTH   2

//Defs for working with the RFID device and storing tag data
#define SERIAL_BUFFER_SIZE 80
#define MAX_TAG_LENGTH 30
char serial_buffer[SERIAL_BUFFER_SIZE];
char tag_id_buf[MAX_TAG_LENGTH];

/*  Prototypes and useful functions */

//Blinks an LED five times - mode is YELLOW, GREEN or BOTH
void blinkled(unsigned char mode);

//Catches a string and writes to the buffer
//In this example the function skips any <CR>, <LF>, '-' or '>'
//at the beginning of the string in order to isolate the tag ID
void RFID_get_data();

//Adds a tag to the database
void catchTag();	

/**************************************************/
/** Code that waits for the user to press the button
**/	
void wait_ch(){ 
	delay(500);
	while( BBIT ){}
	delay(300);	
}

#define WAITBUTTON wait_ch()
/**************************************************/


void setup()
{	
	BUTTONINPUT;	// Init the button

	//Init and turn off the LEDs
	pinMode(YLED, OUTPUT);
	pinMode(GLED, OUTPUT);
	digitalWrite(YLED, LOW);
	digitalWrite(GLED, LOW);

	// Open a serial communication to the connected MOD-RFID device
	Serial.begin(rf_baud);

	//WAITBUTTON;  //wait for the user to press the hardware button
}

void loop(){
	//The RFID device will now continuously scan and for tags
	//and alert you by blinking the LEDs

	catchTag();     //This function waits indefinitely until a tag
                	//number is received via the Serial;
                	//The received number(without newline chars, '>' or '-'
                	//is stored in the global variable tag_id_buf and
                	//will be overwritten when the next one is caught.
	blinkled(BOTH);
        delay(500);     //This delay will ensure you only read the tag once
                        //otherwise the RFID reader will may send several reads a second
}

void catchTag()
{
	//Get the tag within reading distance (<10cm)

	//catch the string containing the tag number
	while(!Serial.available());
	RFID_get_data();

	strcpy(tag_id_buf, serial_buffer);
}

void RFID_get_data()
{
	volatile unsigned char c, c_skip, i=0;

	//Skip any <CR>, <LF>, '-', or '>' before the tag ID
	do{
		c_skip = Serial.read();
	}while(c_skip == 0x0d || c_skip == 0x0a || c_skip == '>' || c_skip == '-');

	serial_buffer[i] = c_skip;
	i++;

	do {
		c = Serial.read();

		// Stop when you meet the <CR> character
		if( c != '\r')
		{	
		serial_buffer[i] = c;
		i++;
		}

	}while( c != '\r' );
	serial_buffer[i] = '\0';

	//Catch the <LF> character
	c = Serial.read();
}

void blinkled(unsigned char mode)
{		
	//Turn off all LEDs
	digitalWrite(YLED, LOW);
	digitalWrite(GLED, LOW);

	char i;

	// Blink
	switch(mode)
	{
	case YELLOW:
		for(i=0; i<4;i++)
		{
			digitalWrite(YLED, digitalRead(YLED)^1); //toggle
			delay(200);
		}
		break;

	case GREEN:
		for(i=0; i<4;i++)
		{
			digitalWrite(GLED, digitalRead(GLED)^1); //toggle
			delay(200);
		}
		break;

	case BOTH:
		for(i=0; i<4;i++)
		{
			digitalWrite(YLED, digitalRead(YLED)^1); //toggle
			digitalWrite(GLED, digitalRead(GLED)^1); //toggle
			delay(200);
		}
		break;
	}				
}

